// src/async.ts
import * as fs from "node:fs";
import Vinyl3 from "vinyl";
import { Buffer as Buffer2 } from "safe-buffer";
import gettextParser2 from "gettext-parser";
import c3 from "ansi-colors";
import cs3 from "color-support";

// src/plugin-error.ts
import * as util from "node:util";
import c from "ansi-colors";
import cs from "color-support";
import { readFileSync } from "node:fs";
c.enabled = Boolean(cs().hasBasic);
var packageJSON = JSON.parse(
  readFileSync(new URL("../package.json", import.meta.url), "utf-8")
);
var PluginError = class {
  message;
  category;
  constructor(message, category = "") {
    this.message = message;
    this.category = category.slice(0, 1).toUpperCase() + category.slice(1).toLowerCase();
  }
  toString() {
    return `${c.cyan(packageJSON.name)}  ${c.bold.red(
      `${this.category}Error`
    )}  ${this.message}`;
  }
  // See: https://nodejs.org/api/util.html#custom-inspection-functions-on-objects
  [util.inspect.custom]() {
    return this.toString();
  }
};
var plugin_error_default = PluginError;

// src/options.ts
import { resolve, relative } from "node:path";

// src/utils.ts
import Vinyl from "vinyl";
var isArray = (value) => {
  return Object.prototype.toString.call(value) === "[object Array]";
};
var isObject = (value) => {
  return Object.prototype.toString.call(value) === "[object Object]";
};
var isString = (value) => {
  return Object.prototype.toString.call(value) === "[object String]";
};
var isBool = (value) => {
  return Object.prototype.toString.call(value) === "[object Boolean]";
};
var isVinyl = (value) => {
  return Vinyl.isVinyl(value);
};
var isArrayOfStrings = (value) => {
  if (!isArray(value))
    return false;
  return Boolean(
    value.reduce((r, v) => isString(v) && r, true)
  );
};
var isArrayOfVinyls = (value) => {
  if (!isArray(value))
    return false;
  return Boolean(value.reduce((r, v) => isVinyl(v) && r, true));
};
var pathLineSort = (a, b) => {
  if (!isString(a) || !isString(b)) {
    throw new Error("pathLineSort: a or b not a string");
  }
  let a_line, b_line;
  [a, a_line] = a.split(":");
  [b, b_line] = b.split(":");
  a = a.replace(/(^\/|\/$)/g, "");
  b = b.replace(/(^\/|\/$)/g, "");
  if (a === b) {
    if ((!a_line || "" === a_line) && b_line && b_line.length)
      return -1;
    if ((!b_line || "" === b_line) && a_line && a_line.length)
      return 1;
    if (a_line && a_line.length && b_line && b_line.length) {
      return parseInt(a_line) - parseInt(b_line);
    }
  }
  const a_ar = a.split("/");
  const b_ar = b.split("/");
  const a_len = a_ar.length;
  const b_len = b_ar.length;
  const l = Math.max(a_len, b_len);
  for (let i = 0; i < l; i++) {
    if (i >= b_len || !b_ar[i])
      return -1;
    if (i >= a_len || !a_ar[i])
      return 1;
    const [a_part, a_ext] = a_ar[i].split(/\.(?=[^.]*$)/g);
    const [b_part, b_ext] = b_ar[i].split(/\.(?=[^.]*$)/g);
    const a_is_file = a_ext && "" !== a_ext;
    const b_is_file = b_ext && "" !== b_ext;
    if (!a_is_file && b_is_file)
      return -1;
    if (a_is_file && !b_is_file)
      return 1;
    if (a_part.toUpperCase() > b_part.toUpperCase())
      return 1;
    if (a_part.toUpperCase() < b_part.toUpperCase())
      return -1;
    if (a_ext && b_ext) {
      if (a_ext.toUpperCase() > b_ext.toUpperCase())
        return 1;
      if (a_ext.toUpperCase() < b_ext.toUpperCase())
        return -1;
    }
  }
  return 0;
};

// src/options.ts
import { isVinyl as isVinyl2 } from "vinyl";
var OptionsError = class extends plugin_error_default {
  constructor(message) {
    super(message, "options");
  }
};
var cwd = "./";
var validateOptionsInput = (options) => {
  if (isObject(options)) {
    if ("potSources" in options && !isString(options.potSources) && !isArrayOfStrings(options.potSources) && // !Buffer.isBuffer(options.potSources) &&
    // !isArrayOfBuffers(options.potSources) &&
    // !isArrayOfVinylsOrBuffers(options.potSources) &&
    !isVinyl2(options.potSources) && !isArrayOfVinyls(options.potSources)) {
      throw new OptionsError(
        "Option potSources should be a string or Vinyl object, or an array of those."
      );
    }
    if ("poSources" in options && // options.poSources &&
    options.poSources !== null && !isString(options.poSources) && !isArrayOfStrings(options.poSources)) {
      throw new OptionsError(
        "Option poSources should be a glob string or glob array."
      );
    }
    if ("wrapLength" in options && (typeof options.wrapLength !== "number" || 0 >= options.wrapLength)) {
      throw new OptionsError(
        "If set, option wrapLength should be a number higher than 0."
      );
    }
    const if_set_string_without_newlines = [
      "srcDir",
      "destDir",
      "domain"
    ];
    for (const k of if_set_string_without_newlines) {
      if (k in options) {
        if (!isString(options[k])) {
          throw new OptionsError(`Option ${k} should be a string.`);
        } else if (options[k].match(/\n/)) {
          throw new OptionsError(
            `Option ${k} can't contain newline characters.`
          );
        }
      }
    }
    const if_set_bool = [
      "writeFiles",
      "returnPOT",
      "domainFromPOTPath",
      "domainInPOPath",
      "defaultContextAsFallback",
      "appendNonIncludedFromPO",
      "includePORevisionDate",
      "includeGenerator",
      "logResults"
    ];
    for (const k of if_set_bool) {
      if (k in options && !isBool(options[k])) {
        throw new OptionsError(`Option ${k} should be a boolean.`);
      }
    }
  } else if (isString(options) || isArrayOfStrings(options)) {
    return {
      poSources: options
    };
  } else if (typeof options !== "undefined") {
    throw new OptionsError(
      "Options should be an object of options, glob string or glob array."
    );
  } else {
    return {};
  }
  return options;
};
var sanitizeAndStandardizeOptionsInput = (options) => {
  if (typeof options.potSources !== "undefined") {
    if (!isArray(options.potSources)) {
      options.potSources = [options.potSources];
    }
    options.potSources = options.potSources.map((v) => isString(v) ? v.trim() : v).filter((v) => !isString(v) || v.length > 0);
  }
  if (typeof options.poSources !== "undefined" && options.poSources !== null) {
    if (!isArray(options.poSources)) {
      options.poSources = [options.poSources];
    }
    options.poSources = options.poSources.map((v) => v.trim()).filter((v) => v.length > 0);
  }
  if (options.srcDir) {
    options.srcDir = resolve(options.srcDir.trim());
    options.srcDir = `${relative(cwd, options.srcDir)}/`.replace(/\\/g, "/").replace(/\/+/g, "/").replace(/(?<!\.)\.\//g, "").replace(/^\//g, "");
  }
  if (options.destDir) {
    options.destDir = resolve(options.destDir.trim());
    options.destDir = `${relative(cwd, options.destDir)}/`.replace(/\\/g, "/").replace(/\/+/g, "/").replace(/(?<!\.)\.\//g, "").replace(/^\//g, "");
  }
  if (options.wrapLength) {
    options.wrapLength = Math.ceil(options.wrapLength);
  }
  return options;
};
var options_default = (options) => {
  cwd = resolve();
  options = validateOptionsInput(options);
  options = sanitizeAndStandardizeOptionsInput(options);
  const defaultOptions = {
    // Input-related
    potSources: ["**/*.pot", "!node_modules/**"],
    poSources: null,
    srcDir: "",
    domainInPOPath: true,
    domainFromPOTPath: true,
    domain: "",
    srcGlobOptions: {},
    // Content-related
    wrapLength: 77,
    defaultContextAsFallback: false,
    appendNonIncludedFromPO: false,
    includePORevisionDate: false,
    includeGenerator: true,
    // Output-related
    returnPOT: false,
    writeFiles: true,
    destDir: "",
    logResults: false
  };
  const resolvedOptions = Object.assign({}, defaultOptions, options);
  if (resolvedOptions.domainFromPOTPath === false && resolvedOptions.domainInPOPath === true && 0 >= resolvedOptions.domain.length) {
    throw new OptionsError(
      "Option domain should be a non-empty string when domainFromPOTPath is false and domainInPOPath is true."
    );
  }
  if (resolvedOptions.returnPOT && !resolvedOptions.writeFiles) {
    throw new OptionsError(
      "If option returnPOT is true, option writeFiles must be true or no PO files will be generated."
    );
  }
  return resolvedOptions;
};

// src/shared.ts
import { basename, dirname } from "node:path";
import { Buffer } from "safe-buffer";
import { sync as matchedSync } from "matched";
import { existsSync, mkdirSync, writeFileSync } from "node:fs";
import gettextParser from "gettext-parser";
import Vinyl2 from "vinyl";
import c2 from "ansi-colors";
import cs2 from "color-support";
import { readFileSync as readFileSync2 } from "node:fs";
c2.enabled = Boolean(cs2().hasBasic);
var packageJSON2 = JSON.parse(
  readFileSync2(new URL("../package.json", import.meta.url), "utf-8")
);
var resolvePOTFilepaths = (options) => {
  if (isArray(options.potSources) && options.potSources.length && isString(options.potSources[0])) {
    options.potSources = matchedSync(options.potSources);
  }
  options._potFilenames = options.potSources.map(
    (f) => isString(f) ? f : f.path
  );
  if (0 >= options.potSources.length) {
    throw new plugin_error_default("No POT files found to process.");
  }
  if (1 < options.potSources.length && options.poSources) {
    throw new plugin_error_default(
      "When processing multiple POT files, leave option poSources empty.\nElse, the same generated PO files will be overwritten for each POT file.",
      "options"
    );
  }
  return options;
};
var getPOFilepaths = (pot_filepath, options) => {
  const pot_name = basename(pot_filepath, ".pot");
  const domain = options.domainFromPOTPath ? pot_name : options.domain;
  const po_dir = options.srcDir ? options.srcDir : `${dirname(pot_filepath)}/`;
  const po_files_glob = [];
  if (options.poSources) {
    po_files_glob.push(...options.poSources);
  } else {
    const locale_glob = "[a-z][a-z]?([a-z])?(_[A-Z][A-Z]?([A-Z]))?(_formal)";
    const domain_glob = options.domainInPOPath ? `${domain}-` : "";
    po_files_glob.push(`${po_dir}${domain_glob}${locale_glob}.po`);
  }
  const po_filepaths = matchedSync(
    po_files_glob,
    options.srcGlobOptions
  ).sort(pathLineSort);
  return po_filepaths;
};
var generatePO = (pot_object, po_object, po_filepath, options) => {
  let new_po_object = JSON.parse(JSON.stringify(pot_object));
  new_po_object = fillPO(new_po_object, po_object, options);
  const new_po_output = compilePO(new_po_object, options);
  const new_po_filepath = basename(po_filepath);
  if (options.writeFiles) {
    writePO(`${options.destDir}${new_po_filepath}`, new_po_output);
  }
  return new Vinyl2({
    contents: Buffer.from(new_po_output),
    path: new_po_filepath
  });
};
var fillPO = (new_po_object, po_object, options) => {
  var _a, _b, _c, _d, _e, _f, _g, _h, _i, _j, _k, _l, _m;
  for (const [ctxt, entries] of Object.entries(new_po_object.translations)) {
    for (const [msgid, entry] of Object.entries(entries)) {
      if (po_object.translations[ctxt] && po_object.translations[ctxt][msgid] && po_object.translations[ctxt][msgid]["msgstr"].length === entry["msgstr"].length) {
        new_po_object.translations[ctxt][msgid]["msgstr"] = [
          ...po_object.translations[ctxt][msgid]["msgstr"]
        ];
      } else if (options.defaultContextAsFallback && po_object.translations[""] && po_object.translations[""][msgid] && po_object.translations[""][msgid]["msgstr"].length === entry["msgstr"].length) {
        new_po_object.translations[ctxt][msgid]["msgstr"] = [
          ...po_object.translations[""][msgid]["msgstr"]
        ];
        new_po_object.translations[ctxt][msgid].comments = {
          ...((_a = new_po_object.translations[ctxt][msgid]) == null ? void 0 : _a.comments) ?? {},
          flag: [
            "fuzzy",
            (_c = (_b = new_po_object.translations[ctxt][msgid]) == null ? void 0 : _b.comments) == null ? void 0 : _c.flag
          ].filter((v) => v).join(", ")
        };
        po_object.translations[""][msgid].comments = {
          ...((_d = po_object.translations[""][msgid]) == null ? void 0 : _d.comments) ?? {},
          translator: [
            `NOTE: re-used for same message, but with context '${ctxt}'`,
            (_f = (_e = po_object.translations[""][msgid]) == null ? void 0 : _e.comments) == null ? void 0 : _f.translator
          ].filter((v) => v).join("\n")
        };
      }
    }
  }
  if (options.appendNonIncludedFromPO) {
    for (const [ctxt, entries] of Object.entries(po_object.translations)) {
      if (!new_po_object.translations[ctxt]) {
        new_po_object.translations[ctxt] = {};
      }
      for (const [msgid, entry] of Object.entries(entries)) {
        if (!new_po_object.translations[ctxt][msgid]) {
          new_po_object.translations[ctxt][msgid] = entry;
          new_po_object.translations[ctxt][msgid].comments = {
            ...((_g = new_po_object.translations[ctxt][msgid]) == null ? void 0 : _g.comments) ?? {},
            ...((_i = (_h = new_po_object.translations[ctxt][msgid]) == null ? void 0 : _h.comments) == null ? void 0 : _i.translator) && ((_j = entry == null ? void 0 : entry.comments) == null ? void 0 : _j.translator.match(/^DEPRECATED$/gm)) ? {} : {
              translator: isString(
                (_l = (_k = new_po_object.translations[ctxt][msgid]) == null ? void 0 : _k.comments) == null ? void 0 : _l.translator
              ) ? "DEPRECATED\n" + (((_m = new_po_object.translations[ctxt][msgid].comments) == null ? void 0 : _m.translator) ?? "") : "DEPRECATED"
            }
          };
        }
      }
    }
  }
  if (options.includePORevisionDate) {
    const d = /* @__PURE__ */ new Date();
    const po_rev_date_string = [
      `${d.getUTCFullYear()}`,
      `-${String(d.getUTCMonth() + 1).padStart(2, "0")}`,
      `-${String(d.getUTCDate()).padStart(2, "0")}`,
      ` ${String(d.getUTCHours()).padStart(2, "0")}`,
      `:${String(d.getUTCMinutes()).padStart(2, "0")}`,
      "+0000"
    ].join("");
    new_po_object.headers["po-revision-date"] = po_rev_date_string;
  }
  if (options.includeGenerator) {
    new_po_object.headers["X-Generator"] = `${packageJSON2.name}/${packageJSON2.version}`;
  }
  return new_po_object;
};
var compilePO = (new_po_object, options) => {
  return gettextParser.po.compile(new_po_object, {
    foldLength: options.wrapLength,
    // Sort entries by first reference filepath and line number.
    sort: (a, b) => {
      var _a, _b, _c, _d, _e, _f;
      const b_deprecated = (_b = (_a = b.comments) == null ? void 0 : _a.translator) == null ? void 0 : _b.match(/^DEPRECATED$/gm);
      const a_deprecated = (_d = (_c = a.comments) == null ? void 0 : _c.translator) == null ? void 0 : _d.match(/^DEPRECATED$/gm);
      if (!a_deprecated && b_deprecated)
        return -1;
      if (a_deprecated && !b_deprecated)
        return 1;
      if (!((_e = b.comments) == null ? void 0 : _e.reference))
        return -1;
      if (!((_f = a.comments) == null ? void 0 : _f.reference))
        return 1;
      a = a.comments.reference.trim().split(/\s+/)[0];
      b = b.comments.reference.trim().split(/\s+/)[0];
      return pathLineSort(a, b);
    }
  });
};
var writePO = (new_po_filepath, new_po_output) => {
  const new_po_dir = dirname(new_po_filepath);
  if (!existsSync(new_po_dir)) {
    mkdirSync(new_po_dir, { recursive: true });
  }
  writeFileSync(new_po_filepath, new_po_output.toString());
};
var logResults = (pots, pos_in, pos_out, dest) => {
  pots.forEach((pot, i) => {
    var _a, _b;
    const pot_filepath = basename(pot);
    const po_filepaths_in = (_a = pos_in[i]) == null ? void 0 : _a.map((po) => po);
    const po_filepaths_out = (_b = pos_out[i]) == null ? void 0 : _b.map((po) => po.path);
    const max_length_in = po_filepaths_in.reduce(
      (p, c4) => Math.max(c4.length, p),
      0
    );
    console.log("");
    if (po_filepaths_out && po_filepaths_out.length) {
      console.log(`  ${c2.bold.green("\u25A0")} ${c2.white(pot_filepath)}`);
      po_filepaths_out.forEach((po_filepath_out, pi) => {
        console.log(
          [
            "    ",
            `${c2.cyan(po_filepaths_in[pi].padEnd(max_length_in, " "))}`,
            ` ${c2.gray("\u2014\u25BA")} `,
            `${c2.yellow(dest)}${c2.yellow(po_filepath_out)}`
          ].join("")
        );
      });
    } else {
      console.log(`  ${c2.gray("\u25A0")} ${c2.white(pot_filepath)}`);
      console.log(`    ${c2.gray("No PO files found.")}`);
    }
  });
  console.log("");
};

// src/async.ts
c3.enabled = Boolean(cs3().hasBasic);
var pot_input_files = [];
var po_input_files = [];
var parsePO = (po_filepath, resolve2, reject) => {
  fs.readFile(po_filepath, (err, file_content) => {
    if (err)
      reject(err);
    const po_object = gettextParser2.po.parse(file_content);
    resolve2(po_object);
  });
};
var processPOT = (pot_file, options, resolve2, reject) => {
  const isVinyl3 = isVinyl(pot_file);
  const pot_filepath = isVinyl3 ? pot_file.path : pot_file;
  const po_filepaths = getPOFilepaths(pot_filepath, options);
  if (po_filepaths.length) {
    if (isVinyl3) {
      if (options.returnPOT) {
        pot_input_files.push(pot_file);
      }
      const pot_object = gettextParser2.po.parse(pot_file.contents);
      resolve2([pot_object, po_filepaths]);
    } else {
      fs.readFile(pot_filepath, (err, pot_content) => {
        if (err)
          reject(err);
        if (options.returnPOT) {
          pot_input_files.push(
            new Vinyl3({
              contents: Buffer2.from(pot_content),
              path: pot_filepath
            })
          );
        }
        const pot_object = gettextParser2.po.parse(pot_content);
        resolve2([pot_object, po_filepaths]);
      });
    }
  } else {
    resolve2(null);
  }
};
var async_default = (cb, options) => {
  if (typeof cb !== "function") {
    throw new plugin_error_default(
      "fillPotPo() requires a callback function as first parameter"
    );
  }
  let resolvedOptions;
  try {
    resolvedOptions = resolvePOTFilepaths(options_default(options));
  } catch (error) {
    cb([false, error.toString()]);
    return;
  }
  pot_input_files = [];
  po_input_files = [];
  Promise.all(
    resolvedOptions.potSources.map((pot_file) => {
      const pot_filepath = isVinyl(pot_file) ? pot_file.relative : pot_file;
      return new Promise(
        (resolve2, reject) => {
          processPOT(pot_file, resolvedOptions, resolve2, reject);
        }
      ).then(async (value) => {
        if (!value) {
          po_input_files.push([]);
          return [];
        }
        const pot_object = value[0];
        const po_files = value[1];
        po_input_files.push(po_files);
        const po_results = await Promise.all(
          po_files.map((po_file) => {
            return new Promise(
              (resolve2, reject) => {
                parsePO(po_file, resolve2, reject);
              }
            ).then((po_object) => {
              return generatePO(
                pot_object,
                po_object,
                po_file,
                resolvedOptions
              );
            }).catch((error) => {
              throw new plugin_error_default(
                `${c3.bold(error.message)}  ${c3.gray(
                  `(PO ${c3.white(po_file)})`
                )}`
              );
            });
          })
        );
        return po_results;
      }).catch((error) => {
        throw new plugin_error_default(
          `${error.message} ${c3.gray(`(POT ${c3.white(pot_filepath)})`)}`
        );
      });
    })
  ).then((po_output_files2) => {
    if (resolvedOptions.logResults) {
      logResults(
        resolvedOptions._potFilenames,
        po_input_files,
        po_output_files2,
        resolvedOptions.destDir
      );
    }
    if (resolvedOptions.returnPOT) {
      cb([true, pot_input_files]);
      return;
    }
    cb([true, [].concat(...po_output_files2)]);
  }).catch((error) => {
    cb([false, error.toString()]);
  });
  return;
};

// src/sync.ts
import * as fs2 from "node:fs";
import Vinyl4 from "vinyl";
import gettextParser3 from "gettext-parser";
var pot_input_files2 = [];
var po_input_files2 = [];
var po_output_files = [];
var processPOs = (po_filepaths, pot_object, options) => {
  const pos = [];
  for (const po_filepath of po_filepaths) {
    const po_content = fs2.readFileSync(po_filepath).toString();
    const po_object = gettextParser3.po.parse(po_content);
    pos.push(generatePO(pot_object, po_object, po_filepath, options));
  }
  po_output_files.push(pos);
};
var processPOT2 = (pot_file, options) => {
  const isVinyl3 = isVinyl(pot_file);
  const pot_filepath = isVinyl3 ? pot_file.path : pot_file;
  const po_filepaths = getPOFilepaths(pot_filepath, options);
  po_input_files2.push(po_filepaths);
  if (po_filepaths.length) {
    let pot_content = "";
    if (isVinyl3) {
      if (options.returnPOT) {
        pot_input_files2.push(pot_file);
      }
      pot_content = pot_file.contents;
    } else {
      const pot_content_buffer = fs2.readFileSync(pot_filepath);
      pot_content = pot_content_buffer.toString();
      if (options.returnPOT) {
        pot_input_files2.push(
          new Vinyl4({
            contents: pot_content_buffer,
            path: pot_filepath
          })
        );
      }
    }
    const pot_object = gettextParser3.po.parse(pot_content);
    processPOs(po_filepaths, pot_object, options);
  } else {
    po_output_files.push([]);
  }
};
var sync_default = (options) => {
  pot_input_files2 = [];
  po_input_files2 = [];
  po_output_files = [];
  const resolvedOptions = resolvePOTFilepaths(options_default(options));
  resolvedOptions.potSources.forEach((pot_file) => {
    processPOT2(pot_file, resolvedOptions);
  });
  if (resolvedOptions.logResults) {
    logResults(
      resolvedOptions._potFilenames,
      po_input_files2,
      po_output_files,
      resolvedOptions.destDir
    );
  }
  if (resolvedOptions.returnPOT) {
    return pot_input_files2;
  }
  return [].concat(...po_output_files);
};

// src/index.ts
var testOptions = {
  wrapLength: 77,
  defaultContextAsFallback: true,
  appendNonIncludedFromPO: true,
  includePORevisionDate: false,
  includeGenerator: false
};
export {
  async_default as default,
  sync_default as sync,
  testOptions
};
